stacksidebar {
  border-left-width: 0px;
  border-right-width: 0px;
}

/* Mimic the status page. */
.avatar-page,
.animation-page {
  margin: 36px 12px;
}

.avatar-page > box > label.title,
.animation-page > box > label.title {
  margin-bottom: 12px;
}

.avatar-page > box > label.description {
  margin-bottom: 36px;
}

#animation-sample {
  min-width: 30px;
  min-height: 30px;
  border-radius: 24px;
  background: linear-gradient(to bottom, var(--green-2), var(--green-5));
}

.tab-page {
  --accent-bg-color: rgb(0 0 0 / 60%);
  --accent-fg-color: white;
  --accent-color: rgb(0 0 0 / 75%);
  background: hsl(from var(--hue) h s 92%);
}

.tab-page-color-1 {
  --hue: var(--blue-3);
}

.tab-page-color-2 {
  --hue: var(--green-3);
}

.tab-page-color-3 {
  --hue: var(--yellow-3);
}

.tab-page-color-4 {
  --hue: var(--orange-3);
}

.tab-page-color-5 {
  --hue: var(--red-3);
}

.tab-page-color-6 {
  --hue: var(--purple-3);
}

.tab-page-color-7 {
  --hue: var(--brown-3);
}

.tab-page-color-8 {
  background: white;
}

.fab {
  margin: 18px;
  padding: 6px;
}
